#! /bin/bash

#shellcheck disable=SC1090,SC1091
source /service-fabric/modules/utils.sh

function test_token_signing_certs()
{
	local namespace="uipath"
	local secret_name=$3
	local identity_cert=$1
	local password=$2
	local tmp_dir
	tmp_dir=$(mktemp -d certificates.XXXXXX)
	if secret=$(kubectl get secret ${secret_name} --namespace ${namespace} -o json 2> /dev/null);
	then
		local password_decoded=$(echo "${secret}" | jq -r ".data.\"$password\"" | base64 -di)
		echo "${secret}" | jq -r ".data.\"$identity_cert\"" | base64 -di | cat > "${tmp_dir}"/"${identity_cert}"
		
		#check file size
		if [ -s "${tmp_dir}"/"${identity_cert}" ];
		then
			# check password
			add_to_report info "Checking ${identity_cert} password ..."
			if [ "${password_decoded}" = "PLACEHOLDER" ]
			then
				add_to_report info "Certificate is not configured ${identity_cert} ..."
				return
			fi
			local result
			result=$(openssl pkcs12 -in "${tmp_dir}"/"${identity_cert}" -nodes --passin pass:${password_decoded} 2> /dev/null)
			if [ ${#result} -gt 0 ]
			then
				add_to_report info "Valid certificate password for ${identity_cert}"

				# check expiration date
				result=$(openssl pkcs12 -in "${tmp_dir}"/"${identity_cert}" -nodes --passin pass:${password_decoded} | openssl x509 -noout -checkend 60)
				if [[ $result == *"not"* ]]
				then
					add_to_report info "Certificate ${identity_cert} is not expired"
				else
					add_to_report error "Certificate ${identity_cert} is expired"
				fi			
			else
				add_to_report error "Invalid certificate password for ${identity_cert}"
			fi
		else
			add_to_report warn "Certificate ${identity_cert} has no value"
		fi
	else
		add_to_report error "Could not fetch certificate ${identity_cert}"
	fi
	rm -rf "$tmp_dir"
}

function test_encryption_keys()
{
	local namespace="uipath"
	local secret_name="identity-generated-secrets"
	local key_name=$1
	local tmp_dir
	tmp_dir=$(mktemp -d keys.XXXXXX)
	if secret=$(kubectl get secret ${secret_name} --namespace ${namespace} -o json 2> /dev/null);
	then
		local key_decoded=$(echo -n "${secret}" | jq -r ".data.\"$key_name\"" | base64 -di)
		# note key needs to be decoded again to get the actual byte[]
		echo -n ${key_decoded} | base64 -di| cat > "${tmp_dir}"/key
		
		# check filesize
		add_to_report info "Checking ${key_name} encryption key ..."
		local filesize=$(cat "${tmp_dir}"/key | wc -c)
		if [ ${filesize} -eq 32 ]
		then
			add_to_report info "Valid encryption key value for ${key_name}"
		else
			add_to_report error "Invalid encryption key value for ${key_name}. Expected size 32 but found ${filesize} "
		fi
    else
		add_to_report error "Could not fetch encryption key ${key_name}"
	fi
	rm -rf "$tmp_dir"
}

function main() 
{
	section "Identity Service Checks"
	test_token_signing_certs "IdentityServer1.pfx" "password1" "identity-token-signing-certificate"
	test_token_signing_certs "IdentityServer2.pfx" "password2" "identity-token-signing-certificate"
	test_token_signing_certs "current.saml2.pfx" "currentCertPassword" "identity-saml-certificates"
	test_token_signing_certs "future.saml2.pfx" "futureCertPassword" "identity-saml-certificates"
	test_encryption_keys "AppSettings__DatabaseProtectionSettings__EncryptionKey2021"
}

main "$@"
exit 0