#!/bin/bash

SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
#shellcheck disable=SC1090,SC1091
source "${SCRIPT_DIR}/../../../modules/utils.sh"

# Prints ceph deployment status
function ceph_deployment_status() {
    section "Ceph Health"
    toolbox_pod=$(kubectl -n rook-ceph get pod -l app=rook-ceph-tools -o jsonpath="{.items[0].metadata.name}")
    CEPH_STATUS=$(kubectl -n rook-ceph exec -i "$toolbox_pod" -- ceph status | grep health | cut -d ':' -f 2 | xargs | sed 's/\r$//')
    CEPH_REASON=$(kubectl -n rook-ceph exec -i "$toolbox_pod" -- ceph status | grep -A 1 health | sed -n 2p | xargs)
    if [[ "${CEPH_STATUS}" = "HEALTH_OK" ]];
    then
        add_to_report info "Ceph status : Healthy"
    elif [[ "${CEPH_STATUS}" = "HEALTH_WARN" ]];
    then
        add_to_report warn "Ceph status : Warning. ${CEPH_REASON}"
    else
        add_to_report error "Ceph status : Failed. ${CEPH_REASON}"
    fi

    add_to_report info "Ceph OSD Stats:"
    add_to_report execute "kubectl -n rook-ceph exec -i $toolbox_pod -- ceph osd status"

    add_to_report info "Ceph Bucket Stats:"
    add_to_report execute "kubectl -n rook-ceph exec -i \"$toolbox_pod\" -- radosgw-admin bucket stats | jq -r '[\"BUCKET-NAME\",\"NO-OF-OBJECTS\",\"SIZE-IN-GB\"], (.[] | [.bucket, .usage.\"rgw.main\".\"num_\
objects\"//0, (((.usage.\"rgw.main\".size_kb_actual//0)/1000000)|.*1000.0 + 0.5|floor/1000.0) ]) | @tsv' | column -ts $'\t'"

    add_to_report info "Ceph PG Stats:"
    add_to_report execute "kubectl -n rook-ceph exec -i \"$toolbox_pod\" -- ceph pg stat"

    gc_pending_count=$(kubectl -n rook-ceph exec -i "$toolbox_pod" -- radosgw-admin gc list --include-all| jq '.[].objs[].oid' | wc -l)
    add_to_report info "Ceph Garbage Collection pending object count: ${gc_pending_count}"

    deployments=$(kubectl -n rook-ceph get deploy --no-headers -o custom-columns=NAME:.metadata.name)
    for deploy in $deployments; do
        check_replicas "$deploy" rook-ceph
    done
    CEPH_OBJECTSTORE_STATUS=$(kubectl get cephobjectstore -n rook-ceph rook-ceph -ojsonpath='{.status.phase}')
    if [[ "${CEPH_OBJECTSTORE_STATUS}" = "Connected" ]];
    then
        add_to_report info "Ceph objectstore status: ${CEPH_OBJECTSTORE_STATUS}"
    else
        add_to_report error "Ceph objectstore status: ${CEPH_OBJECTSTORE_STATUS}"
    fi
    add_to_report info "Ceph Zone info:"
    add_to_report execute "kubectl -n rook-ceph exec -i $toolbox_pod -- radosgw-admin zonegroup get --rgw-zonegroup=rook-ceph --rgw-realm=rook-ceph"
}

function rabbitmq_status() {
    section "RabbitMQ Health"
    DESIRED_REPLICAS=$(kubectl -n rabbitmq get statefulset rabbitmq-server -o jsonpath="{.status.replicas}")
    READY_REPLICAS=$(kubectl -n rabbitmq get statefulset rabbitmq-server -o jsonpath="{.status.readyReplicas}")

    if [[ "${DESIRED_REPLICAS}" == "${READY_REPLICAS}" ]];
    then
        add_to_report info "RabbitMQ all replicas are up."
    else
        add_to_report warn "Not all rabbitMQ replicas are up. ReadyReplicas : ${READY_REPLICAS} DesiredReplicas : ${DESIRED_REPLICAS}"
    fi
}

function mongodb_status() {
    section "MongoDB Health"
    local NAMESPACE=mongodb

    if [ -z "$(kubectl -n $NAMESPACE get deploy -o name)" ];
    then
      add_to_report info "MongoDB is not installed, that's fine if business apps is not enabled"
      return 0
    fi

    USER_NAME=$(kubectl -n $NAMESPACE get secret mongo-admin-user -o json | jq -r '.data.USER_NAME' | base64 -d )
    PASSWORD=$(kubectl -n $NAMESPACE get secret mongo-admin-user -o json | jq -r '.data.PASSWORD' | base64 -d )
    # shellcheck disable=SC2016
    depldbstatus=$(kubectl -n $NAMESPACE exec -i mongodb-replica-set-0 -- sh -c '/usr/bin/mongo --tls --tlsCAFile /var/lib/tls/ca/ca.crt --host `hostname -f` -u '"$USER_NAME"' -p '"'"''"$PASSWORD"''"'"' --authenticationDatabase admin --eval '"'"'db.runCommand("ping").ok'"'"' --quiet' 2>/dev/null)
    # shellcheck disable=SC2016
    depldbprimary=$(kubectl -n $NAMESPACE exec -i mongodb-replica-set-0 -- sh -c '/usr/bin/mongo --tls --tlsCAFile /var/lib/tls/ca/ca.crt --host `hostname -f` -u '"$USER_NAME"' -p '"'"''"$PASSWORD"''"'"' --authenticationDatabase admin --eval '"'"'db.runCommand("hello").primary'"'"' --quiet' 2>/dev/null)

    if [ "$depldbstatus" == 1 ]; 
    then 
      add_to_report info "MongoDBConnectivity: Successful"
    else 
      add_to_report error "MongoDBConnectivity: Failed"
    fi

    if [ -n "$depldbprimary" ]; 
    then 
      add_to_report info "MongoDBPrimaryFound: True" 
    else
      add_to_report error "MongoDBPrimaryFound: False"
    fi
}

function redis_status() {
    section "Redis Health"
    local namespace="redis-system"
    cluster_name=$(get_redis_cluster_name "${namespace}")
    DESIRED_REPLICAS=$(kubectl -n "${namespace}" get statefulset "${cluster_name}" -o jsonpath="{.status.replicas}")
    READY_REPLICAS=$(kubectl -n "${namespace}" get statefulset "${cluster_name}" -o jsonpath="{.status.readyReplicas}")

    if [[ -z "${READY_REPLICAS}" ]];
    then 
        READY_REPLICAS=0
    fi

    if [[ "${DESIRED_REPLICAS}" == "${READY_REPLICAS}" ]];
    then
        add_to_report info "Redis all replicas are up."
    else
        add_to_report warn "Not all Redis replicas are up. ReadyReplicas : ${READY_REPLICAS} DesiredReplicas : ${DESIRED_REPLICAS}"
    fi

    MULTISITE=$(get_cluster_config_key "ACTIVE_ACTIVE.ENABLED")
    PRIMARY=$(get_cluster_config_key "ACTIVE_ACTIVE.PRIMARY")
    if [[ "${MULTISITE:-}" == "true" ]]; then
      add_to_report info "Multisite Setup"
      redis_multisite_checks "${namespace}" "${cluster_name}" "${PRIMARY}"
    fi
}

function redis_multisite_checks() {
    local namespace="${1:-redis-system}"
    local cluster_name="${2:-redis-cluster}"
    local primary="${3:-true}"

    if [[ "${primary}" == "true" ]]; then
      add_to_report info "Primary redis cluster"
    else
      add_to_report info "Secondary redis cluster"
      redis_multisite_replication_check "${namespace}" "${cluster_name}"
    fi
    redis_multisite_cert_checks "${namespace}" "${cluster_name}"
}

function redis_multisite_cert_checks() {
    local namespace="${1:-redis-system}"
    local cluster_name="${2:-redis-cluster}"
    local cluster_fqdn

    cluster_fqdn=$(get_cluster_config_key "ALTERNATIVE_FQDN")
    redis_username=$(kubectl get secret -n "${namespace}" "${cluster_name}" -o jsonpath='{.data.username}' --ignore-not-found | base64 -d)
    redis_password=$(kubectl get secret -n "${namespace}" "${cluster_name}" -o jsonpath='{.data.password}' --ignore-not-found | base64 -d)
    cert_response=$(curl -k -s -u "${redis_username}:${redis_password}" "https://redis.${cluster_fqdn}/v1/cluster/certificates")
    if [[ -n "$cert_response" ]]; then
      for cert_name in "api_cert" "syncer_cert" "ca_cert";
      do
        cert=$(echo "${cert_response}" | jq ".${cert_name}" | sed 's/\\n/\n/g' | tr -d '"')
        cert_expiry=$(echo "$cert" | openssl x509 -noout -enddate | cut -d "=" -f 2)
        if echo "$cert" | openssl x509 -checkend 0 -noout &>/dev/null
        then
          add_to_report info "Redis ${cert_name} certificate not expired. Expiry Date : ${cert_expiry}"
        else
          add_to_report warn "Redis ${cert_name} certificate expired. Expiry Date : ${cert_expiry}"
        fi
      done;
    else
      add_to_report error "Unable to retrieve redis-cluster certificates information"
    fi
}

function redis_multisite_replication_check() {
    local namespace="${1:-redis-system}"
    local cluster_name="${2:-redis-cluster}"
    local redis_db_secret_name="redb-redis-cluster-db"
    local status="failure"
    local remote_kubeconfig_dir
    local remote_kubeconfig_dir_absolute
    local remote_kubeconfig_path
    local try=0
    local maxtry=10
    local wait=10
    local val=""

    remote_kubeconfig_dir=$(mktemp -d config.XXXXXXX)
    remote_kubeconfig_dir_absolute=$(readlink -f "${remote_kubeconfig_dir}")
    remote_kubeconfig_path="${remote_kubeconfig_dir_absolute}/remote-cluster-kubeconfig.yaml"
    redis_db_password=$(kubectl get secret -n "${namespace}" "${redis_db_secret_name}" -o jsonpath='{.data.password}' --ignore-not-found | base64 -d)
    redis_db_port=$(kubectl get secret -n "${namespace}" "${redis_db_secret_name}" -o jsonpath='{.data.port}' --ignore-not-found | base64 -d)
    redis_db_service_name=$(kubectl get secret -n "${namespace}" "${redis_db_secret_name}" -o jsonpath='{.data.service_name}' --ignore-not-found | base64 -d | sed 's/-headless//')
    redis_username=$(kubectl get secret -n "${namespace}" "${cluster_name}" -o jsonpath='{.data.username}' --ignore-not-found | base64 -d)
    redis_password=$(kubectl get secret -n "${namespace}" "${cluster_name}" -o jsonpath='{.data.password}' --ignore-not-found | base64 -d)

    capture_remote_cluster_info "${remote_kubeconfig_path}" "${namespace}" || add_to_report error "Error retrieving remote cluster information or establishing connection"

    # Test replication from remote cluster to current cluster
    if [[ -n "${REMOTE_REC}" ]]; then
      kubectl --kubeconfig="${remote_kubeconfig_path}" -n "${namespace}" exec sts/"${REMOTE_REC}" -c redis-enterprise-node -- redis-cli -h "${redis_db_service_name}" \
          -p "${redis_db_port}" -a "${redis_db_password}" SET dummy_test_key dummy_test_val 2>/dev/null || add_to_report error "Error creating redis database key in the remote cluster"
      while [[ "${status}" != "success" ]] && (( try != maxtry )); do
        try=$((try+1))
        val=$(kubectl -n "${namespace}" exec sts/"${cluster_name}" -c redis-enterprise-node -- redis-cli -h "${redis_db_service_name}" \
          -p "${redis_db_port}" -a "${redis_db_password}" GET dummy_test_key 2>/dev/null || true)
        if [[ "${val}" == "dummy_test_val" ]]; then
          add_to_report info "Redis replication from remote to current cluster is successful"
          status="success"
        else
          echo "Retry...${try}/${maxtry}"
          sleep "${wait}"
        fi
      done
      if [[ -z "${val}" ]]; then
        add_to_report error "Redis replication from remote to current cluster failed"
      fi
      kubectl --kubeconfig="${remote_kubeconfig_path}" -n "${namespace}" exec sts/"${REMOTE_REC}" -c redis-enterprise-node -- redis-cli -h "${redis_db_service_name}" \
          -p "${redis_db_port}" -a "${redis_db_password}" DEL dummy_test_key 2>/dev/null || true

      # Test replication from current cluster to remote cluster
      kubectl -n "${namespace}" exec sts/"${cluster_name}" -c redis-enterprise-node -- redis-cli -h "${redis_db_service_name}" \
          -p "${redis_db_port}" -a "${redis_db_password}" SET dummy_test_key1 dummy_test_val1 2>/dev/null || add_to_report error "Error creating redis database key in the current cluster"
      try=0
      status="failure"
      val=""
      while [[ "${status}" != "success" ]] && (( try != maxtry )); do
        try=$((try+1))
        val=$(kubectl --kubeconfig="${remote_kubeconfig_path}" -n "${namespace}" exec sts/"${REMOTE_REC}" -c redis-enterprise-node -- redis-cli -h "${redis_db_service_name}" \
          -p "${redis_db_port}" -a "${redis_db_password}" GET dummy_test_key1 2>/dev/null || true)
        if [[ "${val}" == "dummy_test_val1" ]]; then
          add_to_report info "Redis replication from current to remote cluster is successful"
          status="success"
        else
          echo "Retry...${try}/${maxtry}"
          sleep "${wait}"
        fi
      done

      if [[ -z "${val}" ]]; then
        add_to_report error "Redis replication from current to remote cluster failed"
      fi

      kubectl -n "${namespace}" exec sts/"${cluster_name}" -c redis-enterprise-node -- redis-cli -h "${redis_db_service_name}" \
          -p "${redis_db_port}" -a "${redis_db_password}" DEL dummy_test_key1 2>/dev/null || true
    fi
    rm -rf "${remote_kubeconfig_dir}"
}

function smb_driver_status() {
    section "SMB Driver Health"

    SMB_DRIVER_STATUS=$(kubectl -n argocd get Application csi-driver-smb --ignore-not-found -o jsonpath="{.status.health.status}")

    if [[ -z "${SMB_DRIVER_STATUS}" ]];
    then
      add_to_report info "Smb driver not enabled."
    else
      CONTROLLER_DESIRED_REPLICAS=$(kubectl -n smb-driver get deployment csi-smb-controller -o jsonpath="{.status.replicas}")
      CONTROLLER_READY_REPLICAS=$(kubectl -n smb-driver get deployment csi-smb-controller -o jsonpath="{.status.readyReplicas}")

      if [[ "${CONTROLLER_DESIRED_REPLICAS}" == "${CONTROLLER_READY_REPLICAS}" ]];
      then
          add_to_report info "Smb driver controller all replicas are up."
      else
          add_to_report warn "Not all smb driver controller replicas are up. ReadyReplicas : ${CONTROLLER_READY_REPLICAS} DesiredReplicas : ${CONTROLLER_DESIRED_REPLICAS}"
      fi

      NODE_DESIRED_SCHEDULED=$(kubectl -n smb-driver get daemonset csi-smb-node -o jsonpath="{.status.desiredNumberScheduled}")
      NODE_NUMBER_READY=$(kubectl -n smb-driver get daemonset csi-smb-node -o jsonpath="{.status.numberReady}")

      if [[ "${NODE_DESIRED_SCHEDULED}" == "${NODE_NUMBER_READY}" ]];
      then
          add_to_report info "Smb driver node daemon all replicas are up."
      else
          add_to_report warn "Not all smb driver node deamon replicas are up. NumberReady : ${NODE_NUMBER_READY} DesiredScheduled : ${NODE_DESIRED_SCHEDULED}"
      fi
    fi
}

function alert_status() {
    section "Prometheus Alerts"
    cluster_ip=$(kubectl get svc rancher-monitoring-prometheus -n cattle-monitoring-system -o jsonpath='{.spec.clusterIP}') || true
    active_alerts=$(curl "http://${cluster_ip}:9090/api/v1/alerts") || true
    filtered_result=""
    echo "$(jq -r '.data.alerts | map(select((.state == "firing") and (.labels.severity == "warning")))' <<< $active_alerts)" | jq -c '.[]' | while read -r alert; 
        do add_to_report warn "Active alert: $alert"; 
    done

    echo "$(jq -r '.data.alerts | map(select((.state == "firing") and (.labels.severity == "critical")))' <<< $active_alerts)" | jq -c '.[]' | while read -r alert; 
        do add_to_report error "Active alert: $alert"; 
    done
}

function airflow_status(){
    section "Airflow Health"
    deployments=$(kubectl -n airflow get deploy --no-headers -o custom-columns=NAME:.metadata.name)
    for deploy in $deployments; do
        check_replicas "$deploy" airflow
    done
}

function is_rook_ceph_installed() {
  if kubectl get cephobjectstore rook-ceph -n rook-ceph &>> "/dev/null"; then
    echo "true"
  else
    echo "false"
  fi
}

alert_status
redis_status
if [[ "$(is_rook_ceph_installed)" == "true" ]]; then
    ceph_deployment_status
fi
rabbitmq_status
mongodb_status
smb_driver_status
airflow_status