#!/bin/bash

SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
#shellcheck disable=SC1090,SC1091
source "${SCRIPT_DIR}/../../../modules/utils.sh"

function get_size_in_GB() {
  local size_in_bytes=$1
  numfmt --to=iec-i "${size_in_bytes}"
}

function loaded_longhorn_modules() {
  required_modules="iscsi_tcp libiscsi_tcp libiscsi scsi_transport_iscsi"
  list_of_loaded_modules=$(lsmod |awk '{print $1}')
  modules_not_found=""
  for module in ${required_modules}; do
    if [[ $(contains "${list_of_loaded_modules}" "${module}") == "false" ]]; then
      modules_not_found="${modules_not_found} ${module}"
    fi
  done
  if [[ -n "${modules_not_found}" ]]; then
    add_to_report error "List of modules not loaded:${modules_not_found}."
  fi
}

function kube_system_deployments(){
    deployments=$(kubectl -n kube-system get deploy --no-headers -o custom-columns=NAME:.metadata.name)
    for deploy in $deployments; do
        check_replicas "$deploy" kube-system
    done
}

function longhorn_system_deployments() {
    deployments=$(kubectl -n longhorn-system get deploy --no-headers -o custom-columns=NAME:.metadata.name)
    for deploy in $deployments; do
        check_replicas "$deploy" longhorn-system
    done
}

function docker_registry_deployments() {
    deployments=$(kubectl -n docker-registry get deploy --no-headers -o custom-columns=NAME:.metadata.name)
    for deploy in $deployments; do
        check_replicas "$deploy" docker-registry
    done
}

function longhorn_nodes_health() {
    section "Longhorn Status"
    longhorn_host=$(kubectl -n longhorn-system get svc longhorn-backend -o jsonpath='{.spec.clusterIP}')
    longhorn_port=$(kubectl -n longhorn-system get svc longhorn-backend -o jsonpath='{.spec.ports[?(@.name=="manager")].port}')
    longhorn_url="http://$longhorn_host:$longhorn_port"
    
    nodes=$(kubectl get nodes --no-headers -o custom-columns=NAME:.metadata.name)
    for node in $nodes; do
        http_response=$(curl -m 5 --noproxy '*' -s -o response.txt -w "%{http_code}" "$longhorn_url"/v1/nodes/"$node")

        if [ "$http_response" = "200" ]; 
        then
          add_to_report info "Longhorn Disk mountPath: $(jq -r '.disks[].path' response.txt)"
          add_to_report info "$node: Schedulable: $(jq -r '.allowScheduling' response.txt)"
          storage_maximum=$(jq -r '.disks[].storageMaximum' response.txt)
          add_to_report info "$node: Maximum Storage: $(get_size_in_GB "$storage_maximum")"
          storage_available=$(jq -r '.disks[].storageAvailable' response.txt)
          add_to_report info "$node: Storage Available: $(get_size_in_GB "$storage_available")"
          storage_scheduled=$(jq -r '.disks[].storageScheduled' response.txt)
          add_to_report info "$node: Storage Scheduled: $(get_size_in_GB "$storage_scheduled")"
          storage_reserved=$(jq -r '.disks[].storageReserved' response.txt)
          add_to_report info "$node: Storage Reserved: $(get_size_in_GB "$storage_reserved")"
          schedulable_size=$(((storage_available - storage_reserved ) * 2))
          add_to_report info "$node: Total Schedulable: $(get_size_in_GB "$schedulable_size")"
          printf "\n"
        else
          add_to_report error "Could not fetch longhorn data for node $node"
        fi
    done
}

function longhorn_volumes_detached_state() {
    DETACHED_VOLUMES=$(kubectl -n longhorn-system get volumes.longhorn.io | grep detached 2>/dev/null)

    if [[ -n "${DETACHED_VOLUMES}" ]];
    then
        section "Longhorn Volumes in Detached state"
        add_to_report execute "kubectl -n longhorn-system get volumes.longhorn.io | grep detached"
    fi
}

section "Infra Deployments"
loaded_longhorn_modules
kube_system_deployments
longhorn_system_deployments
docker_registry_deployments

longhorn_nodes_health
longhorn_volumes_detached_state