#!/bin/bash

export ALL_SUPPORT_BUNDLES_LOCATION=${ALL_SUPPORT_BUNDLES_LOCATION:-/tmp/uipath-support}
SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)

#shellcheck disable=SC1090,SC1091
#importing utility functions and env variable
source $SCRIPT_DIR/utils.sh

function create_bundle_job() {
  local namespace="$1"

  cat << EOF | kubectl create -f - 1>/dev/null
apiVersion: apps/v1
kind: DaemonSet
metadata:
  name: support-root-container
  namespace: ${namespace}
  labels:
    name: support-root-container
    app: support-root-container
spec:
  selector:
    matchLabels:
      name: support-root-container
      app: support-root-container
  template:
    metadata:
      labels:
        name: support-root-container
        app: support-root-container
    spec:
      tolerations:
        - operator: "Exists"
      hostNetwork: true
      containers:
        - name: support-root-container
          image: ${RUNTIME_IMAGE}
          imagePullPolicy: IfNotPresent
          resources:
            limits:
              memory: 256Mi
            requests:
              cpu: 100m
              memory: 100Mi
          securityContext:
            privileged: true
            readOnlyRootFilesystem: true
          volumeMounts:
            - mountPath: /host
              name: noderoot
            - mountPath: /rke2-bundle
              name: scripts
            - mountPath: /host/tmp/bundle-volume
              name: bundle-volume
          command: ["bash"]
          args:
            - -c
            - |
              cp /rke2-bundle/rke2-support-bundle.sh /host/tmp/
              chroot host bash -x /tmp/rke2-support-bundle.sh -d /tmp/bundle-volume
              echo "RKE2 bundle generation completed"
              sleep 100000
      volumes:
        - name: noderoot
          hostPath:
            path: /
        - name: scripts
          configMap:
            name: rke2-support-bundle-scripts
        - name: bundle-volume
          emptyDir: {}
EOF
    if ! kubectl rollout status -n "${namespace}" ds/support-root-container --timeout=3m ; then
        echo "Will try to collect RKE2 support bundle from pods that are available"
    fi
}

function destroy_bundle_job() {
    local namespace=$1
    support_bundle_cm=$(kubectl -n "${namespace}" get cm rke2-support-bundle-scripts --ignore-not-found)
    [[ -n "${support_bundle_cm}" ]] && kubectl -n "${namespace}" delete cm rke2-support-bundle-scripts
    support_bundle_ds=$(kubectl -n "${namespace}" get ds support-root-container --ignore-not-found)
    [[ -n "${support_bundle_ds}" ]] && kubectl -n "${namespace}" delete ds support-root-container
}

function main() {
    local namespace="kube-system"

    destroy_bundle_job "${namespace}"
    set_runtime_image_offline
    kubectl -n ${namespace} create cm rke2-support-bundle-scripts --from-file="${SCRIPT_DIR}/rke2-support-bundle.sh"  --dry-run=client -o yaml | kubectl apply -f -
    create_bundle_job "${namespace}"
    IFS=$'\n' read -r -d '' -a pod_list <<< "$(kubectl -n ${namespace} get pods -l app=support-root-container  -o json |  jq -r '.items[] | select (.metadata.deletionTimestamp==null and .status.phase=="Running") | [.metadata.name] | @tsv')"
    pod_count=${#pod_list[@]}
    if [ "${pod_count}" -eq "0" ]; then
      destroy_bundle_job "${namespace}"
      echo "RKE2 support bundle pods are not ready. Skipping RKE2 bundle"
      return
    fi
    pod_index=0
    while [ "${pod_index}" -lt "${pod_count}" ]; do
        retryCounter=0
        while [ "$retryCounter" -lt 40 ]; do
            kubectl -n ${namespace} logs "${pod_list[${pod_index}]}" | grep "RKE2 bundle generation completed" &> /dev/null || { retryCounter=$((retryCounter+1)) && sleep 10 && continue; }
            bundle_name=$(kubectl -n ${namespace} logs "${pod_list[${pod_index}]}" | grep "Created /tmp/" | tail -1 | cut -d '/' -f4 | tr -d "\r'")
	    [[ -z "${bundle_name}" ]] && break

            kubectl -n ${namespace} cp "${pod_list[$pod_index]}:host/tmp/bundle-volume/${bundle_name}" "${ALL_SUPPORT_BUNDLES_LOCATION}/${bundle_name}"

            if [[ ${bundle_name} =~ .*"tmp.".* ]]; then
               node_name=$(kubectl get -n ${namespace} pods "${pod_list[${pod_index}]}" -o custom-columns=NODE:.spec.nodeName --no-headers)
	       file_name="${node_name}-$(date +'%Y-%m-%d_%H_%M_%S').tar"
               tar --create --file "${ALL_SUPPORT_BUNDLES_LOCATION}/${file_name}" --directory  "${ALL_SUPPORT_BUNDLES_LOCATION}/${bundle_name}" .
	       gzip "${ALL_SUPPORT_BUNDLES_LOCATION:?}/${file_name:?}"
	       rm -rf "${ALL_SUPPORT_BUNDLES_LOCATION:?}/${bundle_name:?}"
            fi
            break
        done
        pod_index=$((pod_index+1))
    done
    destroy_bundle_job "${namespace}"
}


main
