#! /bin/bash

SCRIPT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
if [[ -z "${SCRIPT_DIR}" ]]; then
  error "Could not determine script path"
fi

readonly SCRIPT_DIR

export NAMESPACES_LIST="airflow,argocd,cattle-logging-system,cattle-monitoring-system,cattle-system,cattle-gatekeeper-system,cert-manager,docker-registry,istio-system,kube-system,longhorn-system,rabbitmq,rook-ceph,uipath-infra,uipath,mongodb,redis-system,smb-driver"

#shellcheck disable=SC1090,SC1091
#importing utility functions and env variable
source $SCRIPT_DIR/scripts/utils.sh

export PATH=$PATH:/var/lib/rancher/rke2/bin:${SCRIPT_DIR}/bin:/usr/local/bin
if [[ -z "${KUBECONFIG}" ]];
then
  [[ -f "/etc/rancher/rke2/rke2.yaml" ]] && export KUBECONFIG="${KUBECONFIG:-/etc/rancher/rke2/rke2.yaml}"
  [[ -f "/var/lib/rancher/rke2/agent/kubelet.kubeconfig" ]] && export KUBECONFIG="${KUBECONFIG:-/var/lib/rancher/rke2/agent/kubelet.kubeconfig}"
fi
ARGS=()

export NAMESPACE="uipath-infra"
export EXPORT_ONLY_INFRA_LOGS=0
export LOGS_LOCATION_VM=~/tmp/support-bundle
export ALL_SUPPORT_BUNDLES_LOCATION=/tmp/uipath-support
export EXCLUDE_RKE_BUNDLE="false"

function display_usage() {
  echo "usage: $(basename "$0") [-hd] -N namespaces -F startdate -D numberofdays"
  echo "  -d          Run in debug"
  echo "  -h          Display help"
  echo "  -N          Comma separated list of namespaces from which want to collect the data"
  echo "  -F          Start date to collect the data, YYYY-MM-DD format. If start date is not mentioned, it will collect logs from last 7 days. Ex 2021-06-06"
  echo "  -D          Number of days from the start date for which we want to collect the logs (default value is 2 days)"
  echo "  -k          Generate logs only for infra components"
  echo "  -m          Generate support bundle for mongodb"
  echo "  -o          Output directory (default $ALL_SUPPORT_BUNDLES_LOCATION)"
  echo "  -R          Generate RKE2 support bundle only"
  echo "  -e          Exclude RKE2 support bundle"
  echo "  -s          Exclude redis support bundle"
}

# Parse args utility
function parse_args() {
  while getopts 'hdksmReo:S:P:A:K:N:B:F:D:' flag "$@"; do
    case "${flag}" in
    d)
      info "Running in debug"
      set -x
      ;;
    h)
      display_usage
      exit 0
      ;;
    N)
      export NAMESPACES_LIST=${OPTARG}
      ;;
    F)
      export FROM_DATE=${OPTARG}
      ;;
    D)
      export NUMBER_OF_DAYS=${OPTARG}
      ;;
    k)
      export EXPORT_ONLY_INFRA_LOGS=1
      ;;
    m)
      export GENERATE_MONGO_SUPPORT_BUNDLE=1
      ;;
    R)
      echo "Generating RKE2 support bundle"
      bash scripts/rke2-support-bundle.sh
      exit 0
      ;;
    o)
      [[ -d ${OPTARG} ]] || error "Directory ${OPTARG} not found"
      export ALL_SUPPORT_BUNDLES_LOCATION=${OPTARG}
      ;;
    e)
      export EXCLUDE_RKE_BUNDLE="true"
      ;;
    s)
      export EXCLUDE_REDIS_BUNDLE="true"
      ;;
    :)
      error "Invalid option: ${OPTARG} requires an argument."
      ;;
    *)
      error "Unexpected option ${flag}"
      ;;
    esac
  done
  shift $((OPTIND - 1))
  if [ -z "$FROM_DATE" ];
  then
    FROM_DATE=$(date --date="1 day ago" +%Y-%m-%d)
    NUMBER_OF_DAYS=2
  fi
}

# Gets kubernetes objects yaml files 
# 1 Namespace
# 2 ObjectType - pod, deployment, daemonset etc
function getKubernetesObjects() {
  ns=$1
  objectType=$2
  info "Getting $objectType from $ns namespace"
  OBJECTS=$(kubectl -n "$ns" get "$objectType" --no-headers -o custom-columns=NAME:.metadata.name)
  for obj in $OBJECTS; do
    mkdir -p "${LOGS_LOCATION_VM}/$objectType/$ns/"
    kubectl -n "$ns" get "$objectType" "$obj" -o yaml > "${LOGS_LOCATION_VM}/$objectType/$ns/$obj.yaml"
  done
}

# Collect firing alerts from Prometheus
function collectClusterAlerts() {
  mkdir -p ${LOGS_LOCATION_VM}/alerts/
  info "Getting active alerts from Prometheus"
  cluster_ip=$(kubectl -n cattle-monitoring-system get services/rancher-monitoring-prometheus -o jsonpath='{.spec.clusterIP}') || error_without_exit "Failed to get cluster_ip for rancher-monitoring-prometheus"
  active_alerts=$(curl "http://${cluster_ip}:9090/api/v1/alerts") || error_without_exit "Curl failed while fetching alerts"
  filtered_alerts="$(jq -r '.data.alerts | map(select((.state == "firing") and (.labels.severity == "warning")))' <<< "$active_alerts")" 
  echo "${filtered_alerts}" > ${LOGS_LOCATION_VM}/alerts/warning.json
  filtered_alerts="$(jq -r '.data.alerts | map(select((.state == "firing") and (.labels.severity == "critical")))' <<< "$active_alerts")" 
  echo "${filtered_alerts}" > ${LOGS_LOCATION_VM}/alerts/critical.json
}

# Collect rest of the kuberntes object details, status of deployments, jobs, etc
function collectClusterDetails() {
  collectClusterAlerts || error_without_exit "Failed to fetch active alerts"

  # Get current logs from the pods and different objects yaml files to see the status and any events
  echo "$NAMESPACES_LIST" | tr "," "\n" | while read -r ns
  do
    info "Processing objects in $ns namespace"

    # Get logs from the current states of the pods for the last 4 hours
    PODS=$(kubectl -n "$ns" get pods --no-headers -o custom-columns=NAME:.metadata.name)
    info "Getting current logs from $ns namespace"
    for pod in $PODS; do
      CONTAINERS=$(kubectl -n "$ns" get pod "$pod" -o jsonpath="{.spec.containers[*].name}")
      for container in $CONTAINERS; do
        mkdir -p "${LOGS_LOCATION_VM}/logs/$ns/$pod/"
        kubectl -n "$ns" logs "$pod" "$container" --since=24h >> "${LOGS_LOCATION_VM}/logs/$ns/$pod/$container.log"
      done
      mkdir -p "${LOGS_LOCATION_VM}/pods/$ns/"
      kubectl -n "$ns" get pod "$pod" -o yaml > "${LOGS_LOCATION_VM}/pods/$ns/$pod.yaml"
    done
    
    # Get Kubernets yaml files of different Objects. Deployments, jobs, daemonsets
    getKubernetesObjects "$ns" "deployments"

    getKubernetesObjects "$ns" "jobs"

    getKubernetesObjects "$ns" "daemonsets"

    getKubernetesObjects "$ns" "services"

    mkdir -p ${LOGS_LOCATION_VM}/events/
    kubectl -n "$ns" get events -o json | jq 'del(.items[].metadata.managedFields)' > "${LOGS_LOCATION_VM}/events/$ns.json"
  done
  # get the nodes details
  kubectl describe nodes > ${LOGS_LOCATION_VM}/nodes.json

  # collect Identity Service support bundle 
  uipathNs="uipath"
  mkdir -p "${LOGS_LOCATION_VM}/configmaps/$uipathNs"
  for identity_cm in "identity-service" "identity-service-clients" "identity-service-customconfig"
  do 
    kubectl -n "$uipathNs" get cm "$identity_cm" -o yaml > "${LOGS_LOCATION_VM}/configmaps/$uipathNs/$identity_cm.yaml"
  done

  echo "Support bundle generated on node" > ${LOGS_LOCATION_VM}/rook-ceph-down.txt

  SERVICE_BUNDLE_ZIP="uipath-support-bundle.zip"
  CWD=$(pwd)
  cd ${LOGS_LOCATION_VM} && zip "$ALL_SUPPORT_BUNDLES_LOCATION/$SERVICE_BUNDLE_ZIP" ./* -r && cd "$CWD" || return

  if [[ -n "${LOGS_LOCATION_VM}" && -d "${LOGS_LOCATION_VM}" ]];
  then
    rm -rf ${LOGS_LOCATION_VM}
  fi

  info "Support bundle generated at ${ALL_SUPPORT_BUNDLES_LOCATION}/$SERVICE_BUNDLE_ZIP"
}

function create_mongo_support_bundle() {
  bash "${SCRIPT_DIR}"/scripts/mongodb/fetchdiag.sh "$SCRIPT_DIR"
}

function generate_rke2_support_bundles() {
  bash "${SCRIPT_DIR}/scripts/generate-rke2-support-bundles.sh"
}

function generate_longhorn_support_bundle() {
  ISSUE_URL="https://docs.uipath.com/automation-suite"
  ISSUE_DESCRIPTION="Longhorn Issue"
  BACKEND_URL=$(kubectl -n longhorn-system get svc longhorn-backend -o jsonpath="{.spec.clusterIP}"):9500

  if ! curl "http://${BACKEND_URL}" &>/dev/null;
  then
    info "Longhorn support cannot be generated, as API is unreachable"
    return
  fi

  REQUEST_SUPPORT_BUNDLE=$( curl -sSX POST -H 'Content-Type: application/json' -d '{ "issueURL": "'"${ISSUE_URL}"'", "description": "'"${ISSUE_DESCRIPTION}"'" }' "http://${BACKEND_URL}/v1/supportbundles" )

  ID=$( jq -r '.id' <<< "${REQUEST_SUPPORT_BUNDLE}" )
  SUPPORT_BUNDLE_NAME=$( jq -r '.name' <<< "${REQUEST_SUPPORT_BUNDLE}" )
  echo "Creating support bundle ${SUPPORT_BUNDLE_NAME} on Node ${ID}"
  try=0
  maxtry=300
  while [ "$(curl -sSX GET "http://${BACKEND_URL}/v1/supportbundles/${ID}/${SUPPORT_BUNDLE_NAME}" | jq -r '.state' )" != "ReadyForDownload" ] && (( try != maxtry )); do
    echo "Progress: $(curl -sSX GET "http://${BACKEND_URL}/v1/supportbundles/${ID}/${SUPPORT_BUNDLE_NAME}" | jq -r '.progressPercentage' )%"
    sleep 1s
    try=$((try+1))
  done

  curl -X GET "http://${BACKEND_URL}/v1/supportbundles/${ID}/${SUPPORT_BUNDLE_NAME}/download" --output "${ALL_SUPPORT_BUNDLES_LOCATION}/${SUPPORT_BUNDLE_NAME}.zip"
  echo "Downloaded longhorn support bundle to /tmp/${SUPPORT_BUNDLE_NAME}.zip"
}

function combine_all_bundles() {
  SERVICE_BUNDLE_ZIP="service-fabric-logs-$(date +'%Y-%m-%d-%H-%M-%S').zip"
  CWD=$(pwd)
  cd "${ALL_SUPPORT_BUNDLES_LOCATION}" && zip "/tmp/$SERVICE_BUNDLE_ZIP" ./* -r && cd "$CWD" || return
  info "Support bundle generated at /tmp/$SERVICE_BUNDLE_ZIP"
  export SERVICE_BUNDLE_ZIP
}

function create_service_account() {
  cat << EOF | kubectl apply -f - 1>/dev/null
apiVersion: v1
kind: ServiceAccount
metadata:
  name: support-bundle-generator
  namespace: "${NAMESPACE}"
---
apiVersion: rbac.authorization.k8s.io/v1
kind: ClusterRole
metadata:
  name: uipath-support-bundle-role
rules:
- apiGroups:
  - metrics.k8s.io
  resources:
  - nodes
  verbs:
  - list
  - get
- apiGroups:
  - ""
  resources:
  - pods
  - configmaps
  - pods/log
  - nodes
  - services
  verbs:
  - get
  - list
- apiGroups:
  - apps
  resources:
  - deployments
  - statefulsets
  - daemonsets
  verbs:
  - get
  - list
- apiGroups:
  - batch
  resources:
  - jobs
  verbs:
  - get
  - list
- apiGroups:
  - ""
  resources:
  - events
  verbs:
  - list
- apiGroups:
  - ""
  resources:
  - pods/exec
  verbs:
  - create
---
apiVersion: rbac.authorization.k8s.io/v1
kind: ClusterRoleBinding
metadata:
  name: uipath-support-bundle-generator
roleRef:
  apiGroup: rbac.authorization.k8s.io
  kind: ClusterRole
  name: uipath-support-bundle-role
subjects:
- kind: ServiceAccount
  name: support-bundle-generator
  namespace: "${NAMESPACE}"
EOF
}

function delete_service_account() {
  kubectl -n "${NAMESPACE}" delete ClusterRoleBinding uipath-support-bundle-generator &>/dev/null
  kubectl -n "${NAMESPACE}" delete ClusterRole uipath-support-bundle-role &>/dev/null
  kubectl -n "${NAMESPACE}" delete serviceaccount support-bundle-generator &>/dev/null
}

function populate_common_args() {
  ARGS=()
  if [[ "${STORAGE_TYPE}" == "s3" ]]; then
    use_instance_profile="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.S3_USE_INSTANCE_PROFILE}' | base64 --decode)"
    if [[ "${use_instance_profile}" == "true" ]]; then
      region="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.S3_REGION}' | base64 --decode)"
      bucket="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.BUCKET}' | base64 --decode)"
      
      ARGS+=("--bucket" "${bucket}" "--namespaces" "$NAMESPACES_LIST" "--from-date" "$FROM_DATE" "--days" "$NUMBER_OF_DAYS" "--storage-type" "s3" "--aws-use-instance-profile" "$use_instance_profile" "--aws-region" "${region}")
    else
      access_key="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.S3_ACCESSKEY}' | base64 --decode)"
      secret_key="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.S3_SECRETKEY}' | base64 --decode)"
      fqdn="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.S3_FQDN}' | base64 --decode)"
      port="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.S3_PORT}' | base64 --decode)"
      region="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.S3_REGION}' | base64 --decode)"
      bucket="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.BUCKET}' | base64 --decode)"
      
      ARGS+=("--bucket" "${bucket}" "--aws-host" "$fqdn" "--aws-port" "$port" "--aws-accesskey" "$access_key" "--aws-secretkey" "$secret_key" "--namespaces" "$NAMESPACES_LIST" "--from-date" "$FROM_DATE" "--days" "$NUMBER_OF_DAYS" "--storage-type" "s3" "--aws-use-instance-profile" "$use_instance_profile" "--aws-region" "${region}")
    fi
  elif [[ "${STORAGE_TYPE}" == "azure" ]]; then
    use_managed_identity="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.AZURE_USE_MANAGED_IDENTITY}' | base64 --decode)"
    azure_fqdn_suffix="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.AZURE_FQDN_SUFFIX}' | base64 --decode)"
    if [[ "${use_managed_identity}" == "true" ]]; then
      account_name="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.AZURE_ACCOUNT_NAME}' | base64 --decode)"
      client_id="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.AZURE_CLIENT_ID}' | base64 --decode)"
      bucket="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.BUCKET}' | base64 --decode)"
      
      ARGS+=("--bucket" "${bucket}" "--namespaces" "$NAMESPACES_LIST" "--from-date" "$FROM_DATE" "--days" "$NUMBER_OF_DAYS" "--storage-type" "azure" "--azure-use-managed-identity" "$use_managed_identity" "--azure-accountname" "${account_name}" "--azure-fqdn-suffix" "${azure_fqdn_suffix}")
      [[ "${client_id}" != "CLIENT_ID" ]] && args+=("--azure-clientid" "${client_id}") 
    else
      account_name="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.AZURE_ACCOUNT_NAME}' | base64 --decode)"
      account_key="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.AZURE_ACCOUNT_KEY}' | base64 --decode)"
      bucket="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.BUCKET}' | base64 --decode)"
      
      ARGS+=("--bucket" "${bucket}" "--namespaces" "$NAMESPACES_LIST" "--from-date" "$FROM_DATE" "--days" "$NUMBER_OF_DAYS" "--storage-type" "azure" "--azure-use-managed-identity" "$use_managed_identity" "--azure-accountname" "${account_name}" "--azure-accountkey" "${account_key}" "--azure-fqdn-suffix" "${azure_fqdn_suffix}")
    fi
  else
    BUCKET_NAME='sf-logs'
    OBJECT_GATEWAY_INTERNAL_HOST=$(kubectl -n rook-ceph get services/rook-ceph-rgw-rook-ceph -o jsonpath="{.spec.clusterIP}")
    OBJECT_GATEWAY_INTERNAL_PORT=$(kubectl -n rook-ceph get services/rook-ceph-rgw-rook-ceph -o jsonpath="{.spec.ports[0].port}")
    STORAGE_ACCESS_KEY=$(kubectl -n cattle-logging-system get secret s3-store-secret -o json | jq '.data.OBJECT_STORAGE_ACCESSKEY' | sed -e 's/^"//' -e 's/"$//' | base64 -d)
    STORAGE_SECRET_KEY=$(kubectl -n cattle-logging-system get secret s3-store-secret -o json | jq '.data.OBJECT_STORAGE_SECRETKEY' | sed -e 's/^"//' -e 's/"$//' | base64 -d)
    export OBJECT_GATEWAY_INTERNAL_HOST
    ARGS+=("--bucket" "${BUCKET_NAME}" "--aws-host" "$OBJECT_GATEWAY_INTERNAL_HOST" "--aws-port" "$OBJECT_GATEWAY_INTERNAL_PORT" "--aws-accesskey" "$STORAGE_ACCESS_KEY" "--aws-secretkey" "$STORAGE_SECRET_KEY" "--namespaces" "$NAMESPACES_LIST" "--from-date" "$FROM_DATE" "--days" "$NUMBER_OF_DAYS" "--storage-type" "ceph")
  fi

  [ "${EXCLUDE_REDIS_BUNDLE}" == "true" ] && ARGS+=("--exlude-redis-bundle")
}

function upload_support_bundle_file() {
  filelocation=$1
  chmod 644 "$filelocation" || true
  populate_common_args
  
  ARGS+=("--only-upload" "--host-file-path" "/upload/bundle.zip")

  create_service_account

  delete_support_bundle_job

  kubectl -n ${NAMESPACE} create cm support-bundle-scripts --from-file=scripts/generate-support-bundle.sh --from-file=scripts/azure-managed-signed-url.py --from-file=scripts/azure-non-managed-signed-url.py --from-file=scripts/s3-signed-url.py &>/dev/null
  set_runtime_image_offline

  cat << EOF | kubectl create -f - 1>/dev/null
apiVersion: batch/v1
kind: Job
metadata:
  name: support-bundle-generation
  namespace: ${NAMESPACE}
spec:
  backoffLimit: 0
  template:
    metadata:
      annotations:
        sidecar.istio.io/inject: "false"
      labels:
        app: support-bundle-generation
    spec:
      affinity:
        nodeAffinity:
          requiredDuringSchedulingIgnoredDuringExecution:
            nodeSelectorTerms:
            - matchExpressions:
              - key: kubernetes.io/hostname
                operator: In
                values:
                - $(fetch_hostname_lowercase)
      serviceAccountName: support-bundle-generator
      containers:
      - name: support-bundle
        image: ${RUNTIME_IMAGE}
        imagePullPolicy: Always
        command: ["bash", "/service-fabric/generate-support-bundle.sh"]
        args:
$(
for arg in "${ARGS[@]}"
do
  printf "%10s %s\n"  "-" "\"$arg\""
done
)
        env:
          - name: BASE_PATH
            value: "/service-fabric"
          - name: HTTP_PROXY
            value: "${HTTP_PROXY}"
          - name: HTTPS_PROXY
            value: "${HTTPS_PROXY}"
          - name: NO_PROXY
            value: "${NO_PROXY}"
          - name: http_proxy
            value: "${HTTP_PROXY}"
          - name: https_proxy
            value: "${HTTPS_PROXY}"
          - name: no_proxy
            value: "${NO_PROXY}"            
        securityContext:
          privileged: false
          allowPrivilegeEscalation: false
          runAsUser: 1000
          readOnlyRootFilesystem: true
          capabilities:
            drop: ["NET_RAW"]
        volumeMounts:
        - name: support-bundle-volume
          mountPath: /data
        - name: scripts
          mountPath: /service-fabric
        - name: kubedir
          mountPath: /.kube
        - name: host-mount
          mountPath: /upload/bundle.zip
      restartPolicy: Never
      imagePullSecrets:
      - name: uipathpullsecret
      securityContext:
        runAsUser: 1000
        runAsGroup: 3000
        fsGroup: 2000
      volumes:
      - name: support-bundle-volume
        emptyDir: {}
      - name: scripts
        configMap:
          name: support-bundle-scripts
      - name: kubedir
        emptyDir: {}
      - name: host-mount
        hostPath: 
          path: ${filelocation}
EOF
    retryCounter=1
    podCheck=0
    until [ "$retryCounter" -ge 30 ]
    do
        supportBundlePod=$(kubectl -n ${NAMESPACE} get pods --selector job-name=support-bundle-generation --sort-by=.metadata.creationTimestamp 2>/dev/null | grep support-bundle | tail -n -1)
        supportBundlePodName=$(echo "$supportBundlePod" | awk '{ print $1 }')
	if [[ -z "$supportBundlePodName" ]]; then
          podCheck=$((podCheck+1))
	  [[ $podCheck -ge 5 ]] && {
            warn "Failed to create pod to upload support bundle. Download bundle from VM"
	    break
          }
	  sleep 60
	  continue
        fi
        supportBundlePodStatus=$(kubectl -n ${NAMESPACE} get pod "$supportBundlePodName" -o jsonpath="{.status.phase}")

        if [ "$supportBundlePodStatus" = "Succeeded" ];
        then
            BUNDLE_LINK=$(kubectl -n ${NAMESPACE} logs "$supportBundlePodName" | grep SupportBundleLink | cut -d ' ' -f 2)
            if [ -n "$BUNDLE_LINK" ];
            then
              info "External storage bundle link=${BUNDLE_LINK}"
            fi
            break
        fi

        if [ "$supportBundlePodStatus" = "Failed" ];
        then
          warn "Failed to upload support bundle to remote storage.. Download bundle from VM"
          break
        fi

	if [ "$supportBundlePodStatus" == "Pending" ]; then
          retryCounter=$((retryCounter+1))
          sleep 1m
	  [[ $retryCounter -ge 5 ]] && {
            warn "Failed to schedule a pod to upload support bundle.. Download bundle from VM"
	    break
          }
	fi

	if kubectl -n ${NAMESPACE} get pod "$supportBundlePodName" -o jsonpath="{.status.containerStatuses[*].state.waiting.reason}" | grep -i imagepullbackoff; then
          warn "Failed to upload support bundle because of missing image. Download bundle from VM"
          delete_support_bundle_job
	  break
	fi

        echo -ne "Generating support bundle download link..${progress}\\r"
        progress="${progress}.."
        retryCounter=$((retryCounter+1)) 
        sleep 15
    done
}

# Creates a job that generates support-bundle
function create_support_bundle_job() {
  populate_common_args
  create_service_account

  delete_support_bundle_job

  kubectl -n ${NAMESPACE} create cm support-bundle-scripts --from-file=scripts/ &>/dev/null
  set_runtime_image_offline

  cat << EOF | kubectl create -f - 1>/dev/null
apiVersion: batch/v1
kind: Job
metadata:
  name: support-bundle-generation
  namespace: "${NAMESPACE}"
spec:
  backoffLimit: 0
  template:
    metadata:
      annotations:
        sidecar.istio.io/inject: "false"
    spec:
      serviceAccountName: support-bundle-generator
      containers:
      - name: support-bundle
        image: ${RUNTIME_IMAGE}
        imagePullPolicy: Always
        command: ["bash", "/service-fabric/generate-support-bundle.sh"]
        args:
$(
for arg in "${ARGS[@]}"
do
  printf "%10s %s\n"  "-" "\"$arg\""
done
)
        env:
          - name: BASE_PATH
            value: "/service-fabric"
          - name: HTTP_PROXY
            value: "${HTTP_PROXY}"
          - name: HTTPS_PROXY
            value: "${HTTPS_PROXY}"
          - name: NO_PROXY
            value: "${NO_PROXY}"
          - name: http_proxy
            value: "${HTTP_PROXY}"
          - name: https_proxy
            value: "${HTTPS_PROXY}"
          - name: no_proxy
            value: "${NO_PROXY}"            
        securityContext:
          privileged: false
          allowPrivilegeEscalation: false
          runAsUser: 1000
          readOnlyRootFilesystem: true
          capabilities:
            drop: ["NET_RAW"]
        volumeMounts:
        - name: support-bundle-volume
          mountPath: /data
        - name: scripts
          mountPath: /service-fabric
        - name: kubedir
          mountPath: /.kube
      restartPolicy: Never
      imagePullSecrets:
      - name: uipathpullsecret
      securityContext:
        runAsUser: 1000
        runAsGroup: 3000
        fsGroup: 2000
      volumes:
      - name: support-bundle-volume
        emptyDir: {}
      - name: scripts
        configMap:
          name: support-bundle-scripts
      - name: kubedir
        emptyDir: {} 
EOF
}

function delete_support_bundle_job() {
  kubectl -n ${NAMESPACE} delete job support-bundle-generation &>/dev/null
  kubectl -n ${NAMESPACE} delete cm support-bundle-scripts &>/dev/null
}

function export_cluster_info() {
  ACTIVE_ACTIVE="false"
  PRIMARY="false"
  STORAGE_TYPE="ceph"
  PROXY_ENABLED="false"
  HTTP_PROXY=""
  HTTPS_PROXY=""
  NO_PROXY=""  
  if kubectl get secret external-storage-secrets -n cattle-logging-system &>/dev/null; then
    STORAGE_TYPE="$(kubectl get secret external-storage-secrets -n cattle-logging-system -o jsonpath='{.data.STORAGE_TYPE}' | base64 --decode)"
  fi

  if kubectl get secret service-cluster-configurations -n uipath-infra &>/dev/null; then
    ACTIVE_ACTIVE="$(kubectl get secret service-cluster-configurations -n uipath-infra -o jsonpath='{.data.MULTISITE\.ENABLED}' | base64 -d)"
    PRIMARY="$(kubectl get secret service-cluster-configurations -n uipath-infra -o jsonpath='{.data.MULTISITE\.PRIMARY}' | base64 -d)"
    PROXY_ENABLED="$(kubectl get secret service-cluster-configurations -n uipath-infra -o jsonpath='{.data.PROXY_ENABLED}' | base64 -d)"
    HTTP_PROXY="$(kubectl get secret service-cluster-configurations -n uipath-infra -o jsonpath='{.data.HTTP_PROXY}' | base64 -d)"
    HTTPS_PROXY="$(kubectl get secret service-cluster-configurations -n uipath-infra -o jsonpath='{.data.HTTPS_PROXY}' | base64 -d)"
    NO_PROXY="$(kubectl get secret service-cluster-configurations -n uipath-infra -o jsonpath='{.data.NO_PROXY}' | base64 -d)"
  fi

  export STORAGE_TYPE
  export ACTIVE_ACTIVE
  export PRIMARY
  export PROXY_ENABLED
  export HTTP_PROXY
  export HTTPS_PROXY
  export NO_PROXY 
}

function main() {

    parse_args "$@"
    set_runtime_image_offline

    if ! kubectl -n ${NAMESPACE} get pods &>/dev/null;
    then
        error "Kubectl command not working properly. Please check if current context is set correctly and cluster is responding."
    fi

    export_cluster_info

    if [[ "${GENERATE_MONGO_SUPPORT_BUNDLE}" == "1" ]];
    then
      create_mongo_support_bundle
      exit 0
    fi

    if [[ "${EXPORT_ONLY_INFRA_LOGS}" == "1" ]];
    then
      export NAMESPACES_LIST="default,kube-system,longhorn-system,docker-registry"
      collectClusterDetails
      exit 0
    fi

    rm -rf "${ALL_SUPPORT_BUNDLES_LOCATION}"
    mkdir -p "${ALL_SUPPORT_BUNDLES_LOCATION}"

    create_support_bundle_job

    [[ "${EXCLUDE_RKE_BUNDLE}" == "true" ]] || generate_rke2_support_bundles
    generate_longhorn_support_bundle

    progress=".."
    retryCounter=0
    info "It may take a few mins to generate support bundle. Please wait."
    until [ "$retryCounter" -ge 40 ]
    do
        supportBundlePod=$(kubectl -n ${NAMESPACE} get pods --selector job-name=support-bundle-generation --sort-by=.metadata.creationTimestamp | grep support-bundle | tail -n -1)
        if [ -z "$supportBundlePod" ];
         then
                break
        fi
        supportBundlePodName=$(echo "$supportBundlePod" | awk '{ print $1 }')
        supportBundlePodStatus=$(kubectl -n ${NAMESPACE} get pod "$supportBundlePodName" -o jsonpath="{.status.phase}")

        if [ "$supportBundlePodStatus" = "Succeeded" ];
        then
            BUNDLE_LINK=$(kubectl -n ${NAMESPACE} logs "$supportBundlePodName" | grep SupportBundleLink | cut -d ' ' -f 2)
            if [ -n "$BUNDLE_LINK" ];
            then
                if [[ "${STORAGE_TYPE}" == "ceph" ]]; then
                  info "Bundle generation successful."
                  info "Internal support bundle download link : $BUNDLE_LINK"
                  INGRESS_HOST=$(kubectl -n istio-system get gateway main-gateway -o jsonpath='{.spec.servers[?(@.port.name=="https")].hosts[0]}')
                  if [ -n "${INGRESS_HOST}" ];
                  then
                    EXTERNAL_LINK=$(echo "${BUNDLE_LINK}" | sed "s/http:\/\/${OBJECT_GATEWAY_INTERNAL_HOST}/https:\/\/objectstore.${INGRESS_HOST}/g")
                    info "External support bundle download link : ${EXTERNAL_LINK}"
                  fi
                  curl "$BUNDLE_LINK" -o "${ALL_SUPPORT_BUNDLES_LOCATION}/uipath-support-bundle.zip"
                  combine_all_bundles
                elif [[ "${STORAGE_TYPE}" == "s3" || "${STORAGE_TYPE}" == "azure" ]]; then
                  info "External support bundle download link : ${BUNDLE_LINK}"
                  curl -k "$BUNDLE_LINK" -o "${ALL_SUPPORT_BUNDLES_LOCATION}/uipath-support-bundle.zip"
                  combine_all_bundles  
                fi
            fi
            break
        fi

        if [ "$supportBundlePodStatus" = "Failed" ];
        then
          warn "Support bundle generation job failed. Generating on the VM"
          break
        fi

        if kubectl -n ${NAMESPACE} get pod "$supportBundlePodName" -o jsonpath="{.status.containerStatuses[*].state.waiting.reason}" | grep -i imagepullbackoff; then
          warn "Support bundle generation job failed because of missing image. Generating on the VM"
          break
        fi

        echo -ne "Generating support bundle..${progress}\\r"
        progress="${progress}.."
        retryCounter=$((retryCounter+1)) 
        sleep 15
    done

    if [ -z "$BUNDLE_LINK" ];
    then
        collectClusterDetails
        combine_all_bundles
    fi

    upload_support_bundle_file "/tmp/$SERVICE_BUNDLE_ZIP"

    delete_support_bundle_job
    delete_service_account
    kubectl delete -f "${SCRIPT_DIR}/scripts/node-daemonset-tmp.yaml" &>/dev/null
}

main "$@"
